/*
 ESPLMouseAP.ino - Access point for ESPLineMouse
 
 This program provides an access point within itself, rather than use the existing
 WiFi infrastructure as the original versions of ESPLineMouse do.

 First written 15Nov2019 by JED@pobox.com
 Includes the code from ESPPackU8x8.ino with surprisingly few additions

*/

/* Create a WiFi access point */

#include <ESP8266WiFi.h>
#include <WiFiUDP.h>

// Includes for running display
#include <SPI.h>
#include <U8g2lib.h>
char forDisp[64];  // buffer for building display items

WiFiUDP Udp;

char *ssid = "SSID00";

#define LEFT_M 16     // Left motor connection
#define RIGHT_M 14    //  Right motor connection
#define UDPPORT 4949
#define MAX_PWM 1023

#define BLUELED 2

// where is the display attached? HERE
#define CLOCK_GPIO 5
#define DATA_GPIO 4
/* Constructor */
// This U8X8 version does NOT use ram buffer and only writes part of screen at a time
U8X8_SSD1306_128X64_NONAME_SW_I2C u8x8(CLOCK_GPIO, DATA_GPIO, /* reset=*/ U8X8_PIN_NONE); 

const int SSIDjump = 13;  // input to select random SSID

void setup() {
  pinMode(BLUELED, OUTPUT);
  pinMode(SSIDjump, INPUT_PULLUP);
  
  // ready screen for display
  u8x8.begin(); 
  
  delay(1000);
  // If jumper set, random SSID names
  if (digitalRead(SSIDjump)==0) {
      sprintf(ssid,"SSID%02d",(RANDOM_REG32%100));
    }
  Serial.begin(115200);
  Serial.println();
  Serial.print("Configuring access point...");
  WiFi.softAP(ssid);

  IPAddress myIP = WiFi.softAPIP();
  Serial.print(ssid);
  Serial.print(" IP address: ");
  Serial.println(myIP);

  u8x8.setFont(u8x8_font_8x13B_1x2_f);
  // column x, row y. A display 128x64 pixel has 16 colums and 8 rows.
  sprintf(forDisp, "SSID %s", ssid);
  u8x8.drawString(0,4, forDisp);
  sprintf(forDisp, "%s", myIP.toString().c_str());
  u8x8.draw1x2String(0, 0, forDisp);
  delay(50);

  if(Udp.begin(UDPPORT) == 1){
      Serial.println("UDP connected");
      // initialize pins 
      analogWrite(LEFT_M,MAX_PWM);
      analogWrite(RIGHT_M,MAX_PWM);
    } else {
      Serial.println("UDP NOT connected");  
    }
}

#define PKTSZ 30
int last_l, last_r;
void loop() {
  int packetSize, t_command, f_command, forward, turn, l_motor, r_motor;
  char inPac[PKTSZ+1];     // allow room for 0x00 to terminate string
  packetSize = Udp.parsePacket();
  if(packetSize > 0 ) {
    Serial.println("");
    Serial.print("Received packet of size ");
    Serial.println(packetSize);

    packetSize = Udp.read(inPac, PKTSZ);
    inPac[packetSize]=0;
    Serial.println(inPac);
    if (packetSize == 16) {
      t_command = ((inPac[0]-'0')*1000 + (inPac[1]-'0')*100 + (inPac[2]-'0')*10 + inPac[3]-'0')-1000;
      f_command = ((inPac[4]-'0')*1000 + (inPac[5]-'0')*100 + (inPac[6]-'0')*10 + inPac[7]-'0')-1000;
      if (t_command > 999) t_command = 999; // keep to 3 digits for OLED display
      if (f_command > 999) f_command = 999; // keep to 3 digits for OLED display
      Serial.print("turn command=");
      Serial.print(t_command);
      Serial.print(" forward command=");
      Serial.print(f_command);

      forward=(f_command-500);              // normalize speed forward to amount (forward OR back)
      if (forward < 0) forward=0;           // this robot HAS no back so set to zero
      r_motor=l_motor=forward;              // set both motors to forward "speed" 0-500
      turn=abs(t_command-500);              // compute magnitude of turn
      if (turn > forward) turn = forward;   // limit turn to motor speed == 0
      if (t_command > 500) {                // apply turn to slow correct motor
        r_motor -= turn;
      }
        else l_motor -= turn;
// now motors range from 0 to 500 but the correct range is 0 to 1023. Doubling is close.        

      Serial.print("  turn=");
      Serial.print(turn);
      Serial.print("  l_motor=");
      Serial.print(l_motor);
      Serial.print("  r_motor=");
      Serial.println(r_motor);
        
// sinking not sourcing so reverse, change range from 0-500 to 0-1023
      analogWrite(LEFT_M,1023-(l_motor*1024./500.));
      analogWrite(RIGHT_M,1023-(r_motor*1024./500.));
      
// now only display packet if a LOT different from the display
  if ( abs(last_l-l_motor) > 50 || abs(last_r-r_motor) > 50 ) { // if big change
      Serial.println("change");
      sprintf(inPac, "%03d,%03d", t_command,f_command);  // display UDP commands
      u8x8.setFont(u8x8_font_inr21_2x4_f );
      u8x8.drawString(0,4, inPac);
      last_l=l_motor;
      last_r=r_motor;
      }
    }  
  }
}
